<?php

namespace ypmFrontend;

require_once(dirname(__FILE__).'/PopupChecker.php');

use YpmPopup\Popup;


class DisplayChecker extends PopupChecker
{
	private static $instance;

	public static function instance()
	{
		if (!isset(self::$instance)) {
			self::$instance = new static;
		}

		return self::$instance;
	}

	/**
	 * It checks whether popup should be loaded on the current page.
	 *
	 * @since 1.5.8
	 *
	 * @param int $popupId popup id
	 * @param  object $post page post data
	 *
	 * @return bool
	 *
	 */
	public function isLoadable($popup, $post)
	{
		$this->setPopup($popup);
		$this->setPost($post);

		$isActive = $popup->isActive();
		$allowToLoad = $this->allowToLoad();

		if (!$isActive) {
			$allowToLoad['option_event'] = false;
		}

		return $allowToLoad;
	}

	/**
	 * Decides whether popup data should be loaded or not
	 *
	 * @since 1.5.8
	 *
	 * @return array
	 *
	 */
	private function allowToLoad()
	{
		$isCustomInserted = $this->isCustomInserted();

		$insertedModes = array(
			'attr_event' => false,
			'option_event' => false
		);

		if ($isCustomInserted) {
			$insertedModes['attr_event'] = true;
		}

		$target = $this->divideTargetData();
		$isPostInForbidden = $this->isPostInForbidden($target);

		if ($isPostInForbidden) {
			return $insertedModes;
		}

		$isPermissive = $this->isPermissive($target);

		if ($isPermissive) {
			$insertedModes['option_event'] = $isPermissive;
		}

		return $insertedModes;
	}

	/**
	 * Divide target data to Permissive and Forbidden
	 *
	 * @since 1.5.8
	 *
	 * @return array $popupTargetData
	 *
	 */
	public function divideTargetData()
	{
		$popup = $this->getPopup();
		$targetData = $popup->getOptionValue('ypm-display-settings');
		return $this->divideIntoPermissiveAndForbidden($targetData);
	}

	/**
	 * Check is popup inserted via short code or class attribute
	 *
	 * @since 1.5.8
	 *
	 * @param
	 *
	 * @return bool
	 *
	 */
	private function isCustomInserted()
	{
		$customInsertData = $this->getCustomInsertedData();
		$popup = $this->getPopup();
		// When popup object is empty it's mean popup is not custom inserted
		if (empty($popup)) {
			return false;
		}
		$popupId = $popup->getId();

		return in_array($popupId, $customInsertData);
	}

	/**
	 * Check whether the current page is corresponding to the saved target data
	 *
	 * @since 1.5.8
	 *
	 * @param array $targetData popup saved target data
	 *
	 * @return bool $isSatisfy
	 *
	 */
	public function isSatisfyForParam($targetData)
	{
		$isSatisfy = false;
		$postId = get_queried_object_id();

		if (empty($targetData['key1'])) {
			return $isSatisfy;
		}
		$targetParam = $targetData['key1'];

		$post = $this->getPost();
		if (isset($post) && empty($postId)) {
			$postId = $post->ID;
		}

		if ($targetParam == 'everywhere') {
			return true;
		}

		if (strpos($targetParam, 'all_') === 0) {
			$currentPostType = get_post_type($postId);
			if ('all_'.$currentPostType == $targetParam) {
				$isSatisfy = true;
			}
		}
		else if (strpos($targetParam, 'archive_') === 0) {
			$currentPostType = get_post_type();
			if ($targetParam == $currentPostType.'archive_') {
				if (is_post_type_archive($currentPostType)) {
					$isSatisfy = true;
				}
			}
		}
		else if (strpos($targetParam, 'selected_') === 0) {
			$values = array();

			if (!empty($targetData['key3'])) {
				$values = array_keys($targetData['key3']);
			}

			if (in_array($postId, $values)) {
				$isSatisfy = true;
			}
		}
		else if (strpos($targetParam, 'categories_') === 0 ) {
			$values = array();
			$isSatisfy = false;

			if (!empty($targetData['key3'])) {
				$values = array_values($targetData['key3']);
			}

			global $post;
			// get current all taxonomies of the current post
			$taxonomies = get_post_taxonomies($post);
			foreach ($taxonomies as $taxonomy) {
				// get current post all categories
				$terms = get_the_terms($post->ID, $taxonomy);
				if (!empty($terms)) {
					foreach ($terms as $term) {
						if (empty($term)) {
							continue;
						}
						if (in_array($term->term_id, $values)) {
							$isSatisfy = true;
							break;
						}
					}
				}
			}
		}
		else if ($targetParam == 'post_type' && !empty($targetData['key3'])) {
			$selectedCustomPostTypes = array_values($targetData['key3']);
			$currentPostType = get_post_type($postId);

			if (in_array($currentPostType, $selectedCustomPostTypes)) {
				$isSatisfy = true;
			}
		}
		else if ($targetParam == 'post_category' && !empty($targetData['key3'])) {
			$values = $targetData['key3'];
			$currentPostCategories = get_the_category($postId);
			$currentPostType = get_post_type($postId);
			if (empty($currentPostCategories) && $currentPostType == 'product') {
				$currentPostCategories = get_the_terms($postId, 'product_cat');
			}

			foreach ($currentPostCategories as $categoryName) {
				if (in_array($categoryName->term_id, $values)) {
					$isSatisfy = true;
					break;
				}

			}
		}
		else if ($targetParam == 'page_type' && !empty($targetData['key3'])) {
			$postTypes = $targetData['value'];
			foreach ($postTypes as $postType) {

				if ($postType == 'is_home_page') {
					if (is_front_page() && is_home()) {
						// Default homepage
						$isSatisfy = true;
						break;
					} else if ( is_front_page() ) {
						// static homepage
						$isSatisfy = true;
						break;
					}
				}
				else if (function_exists($postType) && $postType()) {
					$isSatisfy = true;
					break;
				}
			}
		}
		else if ($targetParam == 'page_template' && !empty($targetData['key3'])) {
			$currentPageTemplate = basename(get_page_template());
			if (in_array($currentPageTemplate, $targetData['key3'])) {
				$isSatisfy = true;
			}
		}
		else if ($targetParam == 'post_tags') {
			if (has_tag()) {
				$isSatisfy = true;
			}
		}
		else if ($targetParam == 'post_tags_ids') {
			$tagsObj = wp_get_post_tags($postId);
			$postTagsValues = (array)@$targetData['value'];
			$selectedTags = array_values($postTagsValues);

			foreach ($tagsObj as $tagObj) {
				if (in_array($tagObj->slug, $selectedTags)) {
					$isSatisfy = true;
					break;
				}
			}
		}

		if (!$isSatisfy && do_action('isAllowedForTarget', $targetData, $post)) {
			$isSatisfy = true;
		}

		return $isSatisfy;
	}

	/**
	 * Get custom inserted data
	 *
	 * @since 1.5.8
	 *
	 * @return array $insertedData
	 */
	public function getCustomInsertedData()
	{
		$post = $this->getPost();
		$insertedData = array();
		/**
		 * ToDO check this logic
		 */
		if (isset($post)) {
			$insertedData = Popup::getCustomInsertedDataByPostId($post->ID);
		}

		return $insertedData;
	}
}